#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <avr/iom16.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "mmc.h"
#include "uart.h"
#include "arc4.h"

// stdout replacement
FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);   
// stdin replacement
FILE mystdin = FDEV_SETUP_STREAM( NULL,uart_getchar, _FDEV_SETUP_READ);

#define SECTOR 0x00000001

/** HEX dump a block of data.
*/
void dump(uint8_t * p, uint16_t len)
{
	int i,j;
    printf("\r\n\r\n");
	for (i=0;i<len/16;i++)
	{
		printf("%04x  ",i*16);
		for (j=0;j<16;j++)
			printf("%02x ",p[i*16+j]);
		printf("  ");
		for (j=0;j<16;j++)
			printf("%c", isalpha(p[i*16+j]) ? p[i*16+j] : '.');
		printf("\r\n");
	}
	printf("\r\n\r\n");
}

/* signal handler for external interrupt int0 */
ISR(INT1_vect)
{
	printf("A aparut o intrerupere externa");
}


/** functia de codare
*/
void codeazaSiScrie(uint8_t*sir, uint8_t*parola, uint8_t sector)
{
	register uint8_t len = 0, paslen = 0;
	printf("%s","Introduceti parola (8 caractere maxim):");
	gets(parola);	
	paslen = strlen(parola);
	printf("%s","Introduceti textul de criptat (250 caractere maxim):");
	scanf("%s",sir);	
	len = strlen(sir);
	printf("inainte de codare:\r\n");
	printf("%s %d %s %d\r\n",sir,len,parola,paslen);
	codareARC4(sir,len,parola,paslen);
	printf("dupa codare\r\n");
	printf("%s\r\n",sir);
	printf("mmc_writesector returns %d\r\n", mmc_writesector(sector, sir));
	dump(sir,len);
}

/** functia de decodare
*/
void citesteSiDecodeaza(uint8_t*sir, uint8_t*parola, uint8_t sector)
{
	register uint8_t len = 0, paslen = 0;
	mmc_readsector(sector,sir);	// read a data sector
	len = strlen(sir);
	dump(sir,len);
	printf("%s","Introduceti parola pentru decodare:");
	scanf("%s",parola);	
	paslen = strlen(parola);
	printf("%s %d %s %d\r\n",sir,len,parola,paslen);
	codareARC4(sir,len,parola,paslen);
    printf("%s\r\n",sir);			// dump sector contents
}

/** main function.
*/
int main(void)
{
    // MMC
	uint8_t optiune;
	uint8_t sectorbuffer[250];
	uint8_t parola[8];
	register uint8_t len = 0, paslen = 0;

	//
	// init the uart
	//
	uart_init();
	//sei();
	//
	// init the interrupts
	//
	// Set Pin 6 (PD2) as the pin to use for this example
  	DDRD = 0;
	PORTD = PORTD | 0xFF;

  	// interrupt on INT1 pin on binary value change (sensor triggered) 
  	//MCUCR = (0<<ISC11) | (1<<ISC10);

  	// turn on interrupts!
  	//GICR  |= (1<<INT1);

	
	//
	// setup callbacks for stdio use
	//
 	stdout = &mystdout;
    stdin = &mystdin;

	//
	// say hello
	//
	printf("**  mmc_demo is alive. **\r\n");

	//
	// init mmc card and report status
	//
	mmc_init();
	//printf("mmc_init returns %d\r\n",  );
//start:
	while(1){
	while (optiune != '\n' && optiune != EOF)
	optiune = getchar();
	printf("Vreti sa codati sau sa decodati?(c/d)\r\n");
	fflush(stdin);
	printf("%s\n",stdin);
	//scanf("%c%*[^\n]%*c", &optiune);
	scanf("%c",&optiune);
	printf("\r\n");

	switch(optiune)
	{
		case 'c':
			//
			// write some data into the mmc
			//
			printf("Introduceti textul de criptat (250 caractere maxim):");
			scanf("%s",sectorbuffer);
			len = strlen(sectorbuffer);
			printf("Introduceti parola (8 caractere maxim):");
			scanf("%s",parola);	
			paslen = strlen(parola);
			//printf("inainte de codare:\r\n");
			//printf("%s %d %s %d\r\n",sectorbuffer,len,parola,paslen);
			codareARC4(sectorbuffer,len,parola,paslen);
			//printf("dupa codare\r\n");
			//printf("%s\r\n",sectorbuffer);
			printf("mmc_writesector returns %d\r\n", mmc_writesector(SECTOR, sectorbuffer));
			//dump(sectorbuffer,len);
		break;
		case 'd':
			//
			// read some data from the mmc
			//
    		mmc_readsector(SECTOR,sectorbuffer);	// read a data sector
			len = strlen(sectorbuffer);
			//dump(sectorbuffer,len);
			printf("%s","Introduceti parola pentru decodare:");
			scanf("%s",parola);	
			paslen = strlen(parola);
			//printf("%s %d %s %d\r\n",sectorbuffer,len,parola,paslen);
			codareARC4(sectorbuffer,len,parola,paslen);
		    printf("Sir decodat:%s\r\n",sectorbuffer);			// dump sector contents
		break;
		case 'q':
			return 0;
		break;
		default:
			printf("Optiunea nu e disponibila!\r\n");
			//goto start;
		break;
	}
	}
	return 0;
}
