/** \file uart.h
	Simple uart functionality
*/

#ifndef __UART_H__
#define __UART_H__

#include <inttypes.h>

#define F_CPU 16000000 // Hz
// requires the constant F_CPU to be set to the frequency of the CPU crystal 

//
// UART Default Baud rate calculation 
//
#define UART_BAUD_RATE			9600								 	//!< Default Baudrate
#define UART_BAUD_SELECT		(F_CPU / (UART_BAUD_RATE * 16L) - 1)	//!< Calculation of default baudrate

/** Init UART.
	Setup I/O ports and interrups for the serial link
*/
void 	uart_init( void );

/** Receive a character.
	Get a character from the communications line.
	Will block if no character is available.
	\return Received character
*/
int  	uart_getchar( FILE* f );

/** Send a character.
	Transmit a character on the communications line.
	\param data Character to send
	\return Copy of transmitted character.
*/
int  	uart_putchar( char data, FILE* f );


#endif

